package com.danitheskunk.skunkworks;

import com.danitheskunk.skunkworks.audio.AudioEngine;
import com.danitheskunk.skunkworks.audio.ISample;
import com.danitheskunk.skunkworks.audio.nodes.Mixer;
import com.danitheskunk.skunkworks.audio.nodes.Node;
import com.danitheskunk.skunkworks.audio.nodes.SamplePlayer;
import com.danitheskunk.skunkworks.gfx.IRenderContext;
import com.danitheskunk.skunkworks.gfx.IRenderContext3D;
import com.danitheskunk.skunkworks.gfx.ITexture;
import com.danitheskunk.skunkworks.gfx.font.IFont;
import com.danitheskunk.skunkworks.nodes.NodeRoot;
import org.lwjgl.glfw.GLFW;

public abstract class BaseGame {
	protected AudioEngine audioEngine;
	protected IFont debugFont;
	protected Engine engine;
	protected Mixer mixer;
	protected NodeRoot rootNode;
	protected SamplePlayer samplePlayer;
	protected IWindow window;

	public BaseGame(Vec2i windowSize, String windowTitle) {
		this.audioEngine = new AudioEngine(44100, 256, 8);
		this.mixer = new Mixer(this.audioEngine, 2);
		this.samplePlayer = new SamplePlayer(this.audioEngine);
		Node.connect(samplePlayer, 0, mixer, 0);
		this.audioEngine.setNode(this.mixer);

		this.engine = new Engine();
		this.window = engine.openWindow(windowSize, windowTitle);
		//todo: load from .jar
		this.debugFont = window.loadFontTileset("fonts/ega-8x14.png");
		this.rootNode = new NodeRoot();
		init();
	}

	protected void init() {

	}

	protected ISample loadSample(String path) {
		return audioEngine.loadSample(path);
	}

	protected ITexture loadTexture(String path) {
		return window.loadTexture(path);
	}

	protected void playSample(ISample sample) {
		samplePlayer.play(sample);
	}

	protected void playSample(ISample sample, boolean looping) {
		samplePlayer.play(sample, looping);
	}

	protected void render(IRenderContext rc) {
		//rc.drawString(new Vec2i(8, 8), "Welcome to Skunkworks, please
		// overide the render method to get started", debugFont, Color
		// .LIGHT_GRAY);
	}

	protected void render3D(IRenderContext3D rc3d) {
	}

	protected void renderPre(IRenderContext rc) {

	}

	public void run() {
		double lastTime, currentTime, delta;

		lastTime = GLFW.glfwGetTime();
		while(!window.shouldClose()) {
			currentTime = GLFW.glfwGetTime();
			delta = currentTime - lastTime;
			lastTime = currentTime;
			engine.tick();
			window.tick();
			//todo: frame rate control
			rootNode.tick();
			update(1.0 / 60.0);
			var rc3d = window.renderStart3D();
			render3D(rc3d);
			window.renderFinish3D(rc3d);
			var rc = window.renderStart();
			renderPre(rc);
			rootNode.render(rc);
			render(rc);
			window.renderFinish(rc);

			window.runScaler();
			audioEngine.refill();
		}
	}

	protected void update(double delta) {

	}
}
